// Date.h
// cs7s14
#include <iostream>
#include <string>
#include <cassert>

using namespace std;
//using std::cout;
//using std::cin;
//using std::endl;
//using std::istream;
//using std::ostream;

const int FIRST_YEAR = 1;
const int LAST_YEAR = 10000;

//enum class Month {January = 1, February, March, April, May, June,
//            July, August, September, October, November, December};

//enum Era {BC, AD};

class Date {
    friend istream& operator>>(istream&, Date&);
    friend ostream& operator<<(ostream&, const Date&);
    friend bool operator==(const Date&, const Date&);
    friend bool operator!=(const Date&, const Date&);
    friend bool operator<(const Date&, const Date&);
    friend bool operator>(const Date&, const Date&);
    friend bool operator<=(const Date&, const Date&);
    friend bool operator>=(const Date&, const Date&);
    friend Date operator+(const Date&, const int);
    friend Date operator-(const Date&, const int);
    friend int operator-(const Date&, const Date&);
public:
    //Date(int m, int d, int y, bool e) : _month(m), _day(d), _year(y), _era(e) {}
    Date() {}
    ~Date() {};
    //void print();
    Date(int =1, int =1, int =1);
    int day_of_year() const;
    int year() const;
    int month() const;
    int day() const;
    int era() const;
    int days() const;
    string weekday() const;
    Date operator++();
    Date operator--();
    Date& operator+=(const int);
    Date& operator-=(const int);
private:
    int _day; //days since FIRST_YEAR
};

bool isLeap(int y) {
    if(y%400 == 0) return true;
    if(y%100 == 0) return false;
    if(y%4 == 0) return true;
    return false;
}

int days_in_month(int m, int y) {
    if (m == 2) return (isLeap(y) ? 29 : 28);
    if (m == 4 || m == 6 || m == 9 || m == 11) return 30;
    return 31;
}

int elapsed(int y, int m, int d)
// returns the number of days between 0-1-1 and y-m-d
{
    int n = 0;
    for (int yy=FIRST_YEAR; yy<y; yy++)
        n += (isLeap(yy) ? 366 : 365);
    for (int mm=1; mm<m; mm++)
        n += days_in_month(mm, y);
    return n + d;
}



istream& operator>>(istream& istr, Date& x)
{ int year, month, day;
  char c;  // to eat the '-'
  cin >> year >> c >> month >> c >> day;
  x._day = elapsed(year, month, day);
  return istr;
}

ostream& operator<<(ostream& ostr, const Date& x) {
    switch(x.month()) {
        case 1 : ostr << "January"; break;
        case 2 : ostr << "February"; break;
        case 3 : ostr << "March"; break;
        case 4 : ostr << "April"; break;
        case 5 : ostr << "May"; break;
        case 6 : ostr << "June"; break;
        case 7 : ostr << "July"; break;
        case 8 : ostr << "August"; break;
        case 9 : ostr << "September"; break;
        case 10 : ostr << "October"; break;
        case 11 : ostr << "November"; break;
        case 12 : ostr << "December"; break;
    }
    ostr << " ";
    //ostr << x.year() << "-";
    //int m = x.month();
    //if (m < 10) ostr << "0";
    //ostr << m << "-";
    //int d = x.day();
    ostr << x.day() << ", ";
    //if (d < 10) ostr << "0";
    //ostr << d;
    ostr << x.year();;// << " " << x.era();
    return ostr;
}

inline bool operator==(const Date& x, const Date& y) {
    return bool(x._day == y._day);
}

inline bool operator!=(const Date& x, const Date& y) {
    return bool(x._day != y._day);
}

inline bool operator<(const Date& x, const Date& y) {
    return bool(x._day < y._day);
}

inline bool operator<=(const Date& x, const Date& y) {
    return bool(x._day <= y._day);
}

inline bool operator>(const Date& x, const Date& y) {
    return bool(x._day > y._day);
}

inline bool operator>=(const Date& x, const Date& y) {
    return bool(x._day >= y._day);
}

inline Date operator+(const Date& x, const int n) {
    Date y = x;
    y._day += n;
    return y;
}

inline Date operator-(const Date& x, const int n) {
    Date y = x;
    y._day -= n;
    return y;
}

inline int operator-(const Date& x, const Date& y) {
    return x._day - y._day;
}

Date::Date(int m, int d, int y) {
    assert(y >= FIRST_YEAR);
    assert(1 <= m && m <= 12);
    assert(1 <= d && d <= days_in_month(m, y));
    _day = elapsed(y, m, d);
}

int Date::day_of_year() const {
    int diy;  // number of days in year
    int n = _day;
    for (int y=FIRST_YEAR; y<LAST_YEAR; y++) {
        diy = (isLeap(y) ? 366 : 365);
        if (n <= diy) return n;
        n -= diy;
  }
  return n;
}

int Date::year() const {
    int diy;  // number of days in year
    int n = _day;
    for (int y=FIRST_YEAR; y<LAST_YEAR; y++) {
        diy = (isLeap(y) ? 366 : 365);
        if (n <= diy) return y;
        n -= diy;
    }
    return LAST_YEAR;
}

int Date::month() const {
    int dim;  // number of days in month
    int y = year();
    int n = day_of_year();
    for (int m=1; m<12; m++)   {
        dim = days_in_month(m, y);
        if (n <= dim) return m;
        n -= dim;
    }
    return 12;
}

int Date::day() const { // day of the month
    int dim;  // number of days in month
    int y = year();
    int n = day_of_year();
    for (int m=1; m<12; m++) {
        dim = days_in_month(m, y);
        if (n <= dim) return n;
        n -= dim;
    }
    return n;
}

inline int Date::days() const {
    return _day;
}

string Date::weekday() const
// Zeller's Algorithm (see Reilly & Federighi, p.309)
{
    const string WEEKDAY[7] = { "Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat" };
    int y = year();
    int m = month();
    int d = days();
    m = 1 + (m + 9)%12;
    if (m > 10) --y;
    int c = y/100;
    y %= 100;
    int i = ((13*m - 1)/5 + d + y + y/4 + c/4 + 5*c)%7;
    return WEEKDAY[i];
}


inline Date Date::operator++() {
    ++_day;
    return *this;
}

inline Date Date::operator--() {
    --_day;
    return *this;
}

inline Date& Date::operator+=(const int n) {
    _day += n;
    return *this;
}

inline Date& Date::operator-=(const int n) {
    _day -= n;
    return *this;
}
